@@ -48,6 +48,8 @@ PyObject* diagonalize(PyObject *self, PyObject *args);
 PyObject* diagonalize_mr3(PyObject *self, PyObject *args);
 PyObject* general_diagonalize(PyObject *self, PyObject *args);
 PyObject* inverse_cholesky(PyObject *self, PyObject *args);
+PyObject* banded_cholesky(PyObject* self, PyObject* args);
+PyObject* solve_banded_cholesky(PyObject* self, PyObject* args);
 PyObject* inverse_symmetric(PyObject *self, PyObject *args);
 PyObject* inverse_general(PyObject *self, PyObject *args);
 PyObject* linear_solve_band(PyObject *self, PyObject *args);
@@ -59,8 +61,7 @@ PyObject* NewWOperatorObject(PyObject *self, PyObject *args);
 PyObject* NewSplineObject(PyObject *self, PyObject *args);
 PyObject* NewTransformerObject(PyObject *self, PyObject *args);
 PyObject* pc_potential(PyObject *self, PyObject *args);
-PyObject* heap_mallinfo(PyObject *self);
-PyObject* elementwise_multiply_add(PyObject *self, PyObject *args);
+PyObject* add_to_density(PyObject *self, PyObject *args);
 PyObject* utilities_gaussian_wave(PyObject *self, PyObject *args);
 PyObject* utilities_vdot(PyObject *self, PyObject *args);
 PyObject* utilities_vdot_self(PyObject *self, PyObject *args);
@@ -78,6 +79,8 @@ PyObject* exterior_electron_density_region(PyObject *self, PyObject *args);
 PyObject* plane_wave_grid(PyObject *self, PyObject *args);
 PyObject* tci_overlap(PyObject *self, PyObject *args);
 PyObject *pwlfc_expand(PyObject *self, PyObject *args);
+PyObject *pw_insert(PyObject *self, PyObject *args);
+PyObject *pw_precond(PyObject *self, PyObject *args);
 PyObject* overlap(PyObject *self, PyObject *args);
 PyObject* vdw(PyObject *self, PyObject *args);
 PyObject* vdw2(PyObject *self, PyObject *args);
@@ -111,6 +114,18 @@ PyObject* pblas_hemm(PyObject *self, PyObject *args);
 PyObject* pblas_gemv(PyObject *self, PyObject *args);
 PyObject* pblas_r2k(PyObject *self, PyObject *args);
 PyObject* pblas_rk(PyObject *self, PyObject *args);
+#if defined(GPAW_WITH_ELPA)
+#include <elpa/elpa.h>
+PyObject* pyelpa_allocate(PyObject *self, PyObject *args);
+PyObject* pyelpa_set(PyObject *self, PyObject *args);
+PyObject* pyelpa_set_comm(PyObject *self, PyObject *args);
+PyObject* pyelpa_setup(PyObject *self, PyObject *args);
+PyObject* pyelpa_diagonalize(PyObject *self, PyObject *args);
+PyObject* pyelpa_general_diagonalize(PyObject *self, PyObject *args);
+PyObject* pyelpa_hermitian_multiply(PyObject *self, PyObject *args);
+PyObject* pyelpa_constants(PyObject *self, PyObject *args);
+PyObject* pyelpa_deallocate(PyObject *self, PyObject *args);
+#endif // GPAW_WITH_ELPA
 #endif // GPAW_WITH_SL and PARALLEL
 
 #ifdef GPAW_PAPI
@@ -168,6 +183,8 @@ static PyMethodDef functions[] = {
     {"diagonalize_mr3", diagonalize_mr3, METH_VARARGS, 0},
     {"general_diagonalize", general_diagonalize, METH_VARARGS, 0},
     {"inverse_cholesky", inverse_cholesky, METH_VARARGS, 0},
+    {"banded_cholesky", banded_cholesky, METH_VARARGS, 0},
+    {"solve_banded_cholesky", solve_banded_cholesky, METH_VARARGS, 0},
     {"inverse_symmetric", inverse_symmetric, METH_VARARGS, 0},
     {"inverse_general", inverse_general, METH_VARARGS, 0},
     {"linear_solve_band", linear_solve_band, METH_VARARGS, 0},
@@ -178,14 +195,15 @@ static PyMethodDef functions[] = {
     {"WOperator", NewWOperatorObject, METH_VARARGS, 0},
     {"Spline", NewSplineObject, METH_VARARGS, 0},
     {"Transformer", NewTransformerObject, METH_VARARGS, 0},
-    {"heap_mallinfo", (PyCFunction) heap_mallinfo, METH_NOARGS, 0},
-    {"elementwise_multiply_add", elementwise_multiply_add, METH_VARARGS, 0},
+    {"add_to_density", add_to_density, METH_VARARGS, 0},
     {"utilities_gaussian_wave", utilities_gaussian_wave, METH_VARARGS, 0},
     {"utilities_vdot", utilities_vdot, METH_VARARGS, 0},
     {"utilities_vdot_self", utilities_vdot_self, METH_VARARGS, 0},
     {"eed_region", exterior_electron_density_region, METH_VARARGS, 0},
     {"plane_wave_grid", plane_wave_grid, METH_VARARGS, 0},
     {"pwlfc_expand", pwlfc_expand, METH_VARARGS, 0},
+    {"pw_insert", pw_insert, METH_VARARGS, 0},
+    {"pw_precond", pw_precond, METH_VARARGS, 0},
     {"erf", errorfunction, METH_VARARGS, 0},
     {"cerf", cerf, METH_VARARGS, 0},
     {"pack", pack, METH_VARARGS, 0},
@@ -235,6 +253,17 @@ static PyMethodDef functions[] = {
     {"pblas_gemv", pblas_gemv, METH_VARARGS, 0},
     {"pblas_r2k", pblas_r2k, METH_VARARGS, 0},
     {"pblas_rk", pblas_rk, METH_VARARGS, 0},
+#if defined(GPAW_WITH_ELPA)
+    {"pyelpa_allocate", pyelpa_allocate, METH_VARARGS, 0},
+    {"pyelpa_set", pyelpa_set, METH_VARARGS, 0},
+    {"pyelpa_setup", pyelpa_setup, METH_VARARGS, 0},
+    {"pyelpa_set_comm", pyelpa_set_comm, METH_VARARGS, 0},
+    {"pyelpa_diagonalize", pyelpa_diagonalize, METH_VARARGS, 0},
+    {"pyelpa_general_diagonalize", pyelpa_general_diagonalize, METH_VARARGS, 0},
+    {"pyelpa_hermitian_multiply", pyelpa_hermitian_multiply, METH_VARARGS, 0},
+    {"pyelpa_constants", pyelpa_constants, METH_VARARGS, 0},
+    {"pyelpa_deallocate", pyelpa_deallocate, METH_VARARGS, 0},
+#endif // GPAW_WITH_ELPA
 #endif // GPAW_WITH_SL && PARALLEL
 #ifdef GPAW_HPM
     {"hpm_start", ibm_hpm_start, METH_VARARGS, 0},
@@ -475,14 +504,28 @@ main(int argc, char **argv)
     Py_SetProgramName(wargv[0]);
     PyImport_AppendInittab("_gpaw", &moduleinit0);
     Py_Initialize();
-
     PySys_SetArgvEx(argc, wargv, 0);
+
+#ifdef GPAW_WITH_ELPA
+    // Globally initialize Elpa library if present:
+    if (elpa_init(20171201) != ELPA_OK) {
+        // What API versions do we support?
+        PyErr_SetString(PyExc_RuntimeError, "Elpa >= 20171201 required");
+        PyErr_Print();
+        return 1;
+    }
+#endif
+
     int status = gpaw_main();
 
     if(status != 0) {
         PyErr_Print();
     }
 
+#ifdef GPAW_WITH_ELPA
+    elpa_uninit();
+#endif
+
     Py_Finalize();
     MPI_Finalize();
 
