@@ -19,10 +19,19 @@
 #endif
 
 // Check that array is well-behaved and contains data that can be sent.
-#define CHK_ARRAY(a) if ((a) == NULL || !PyArray_Check(a)               \
+#define CHK_ARRAY(a) if ((a) == NULL || !PyArray_Check(a)                   \
                          || !PyArray_ISCARRAY(a) || !PyArray_ISNUMBER(a)) { \
-    PyErr_SetString(PyExc_TypeError,                                    \
-                    "Not a proper NumPy array for MPI communication."); \
+    PyErr_SetString(PyExc_TypeError,                                        \
+                    "Not a proper NumPy array for MPI communication.");     \
+    return NULL; }
+
+// Check that array is well-behaved, read-only  and contains data that
+// can be sent.
+#define CHK_ARRAY_RO(a) if ((a) == NULL || !PyArray_Check(a)                \
+                         || !PyArray_ISCARRAY_RO(a)                         \
+                         || !PyArray_ISNUMBER(a)) {                         \
+    PyErr_SetString(PyExc_TypeError,                                        \
+                    "Not a proper NumPy array for MPI communication.");     \
     return NULL; }
 
 // Check that two arrays have the same type, and the size of the
@@ -72,7 +81,7 @@ static PyObject *mpi_request_wait(GPAW_MPI_Request *self, PyObject *noargs)
   int ret = MPI_Wait(&(self->rq), MPI_STATUS_IGNORE);
   if (ret != MPI_SUCCESS)
     {
-      PyErr_SetString(PyExc_RuntimeError, "MPI_Wait error occured.");
+      PyErr_SetString(PyExc_RuntimeError, "MPI_Wait error occurred.");
       return NULL;
     }
 #endif
@@ -96,7 +105,7 @@ static PyObject *mpi_request_test(GPAW_MPI_Request *self, PyObject *noargs)
   int ret = MPI_Test(&(self->rq), &flag, MPI_STATUS_IGNORE); // Can this change the Python string?
   if (ret != MPI_SUCCESS)
     {
-      PyErr_SetString(PyExc_RuntimeError, "MPI_Test error occured.");
+      PyErr_SetString(PyExc_RuntimeError, "MPI_Test error occurred.");
       return NULL;
     }
 #endif
@@ -238,7 +247,7 @@ static PyObject * mpi_sendreceive(MPIObject *self, PyObject *args,
                            PyArray_BYTES(b), nrecv, MPI_BYTE, src, recvtag,
                            self->comm, MPI_STATUS_IGNORE);
     if (ret != MPI_SUCCESS) {
-        PyErr_SetString(PyExc_RuntimeError, "MPI_Sendrecv error occured.");
+        PyErr_SetString(PyExc_RuntimeError, "MPI_Sendrecv error occurred.");
         return NULL;
     }
 #endif
@@ -272,7 +281,7 @@ static PyObject * mpi_receive(MPIObject *self, PyObject *args, PyObject *kwargs)
                          MPI_STATUS_IGNORE);
       if (ret != MPI_SUCCESS)
         {
-          PyErr_SetString(PyExc_RuntimeError, "MPI_Recv error occured.");
+          PyErr_SetString(PyExc_RuntimeError, "MPI_Recv error occurred.");
           return NULL;
         }
 #endif
@@ -291,7 +300,7 @@ static PyObject * mpi_receive(MPIObject *self, PyObject *args, PyObject *kwargs)
                           &(req->rq));
       if (ret != MPI_SUCCESS)
         {
-          PyErr_SetString(PyExc_RuntimeError, "MPI_Irecv error occured.");
+          PyErr_SetString(PyExc_RuntimeError, "MPI_Irecv error occurred.");
           return NULL;
         }
 #endif
@@ -322,7 +331,7 @@ static PyObject * mpi_send(MPIObject *self, PyObject *args, PyObject *kwargs)
       int ret = MPI_Send(PyArray_BYTES(a), n, MPI_BYTE, dest, tag, self->comm);
       if (ret != MPI_SUCCESS)
         {
-          PyErr_SetString(PyExc_RuntimeError, "MPI_Send error occured.");
+          PyErr_SetString(PyExc_RuntimeError, "MPI_Send error occurred.");
           return NULL;
         }
 #endif
@@ -341,7 +350,7 @@ static PyObject * mpi_send(MPIObject *self, PyObject *args, PyObject *kwargs)
                           &(req->rq));
       if (ret != MPI_SUCCESS)
         {
-          PyErr_SetString(PyExc_RuntimeError, "MPI_Isend error occured.");
+          PyErr_SetString(PyExc_RuntimeError, "MPI_Isend error occurred.");
           return NULL;
         }
 #endif
@@ -358,7 +367,7 @@ static PyObject * mpi_ssend(MPIObject *self, PyObject *args, PyObject *kwargs)
   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Oi|i:send", kwlist,
                                    &a, &dest, &tag))
     return NULL;
-  CHK_ARRAY(a);
+  CHK_ARRAY_RO(a);
   CHK_OTHER_PROC(dest);
   int n = PyArray_DESCR(a)->elsize;
   for (int d = 0; d < PyArray_NDIM(a); d++)
@@ -438,7 +447,7 @@ static PyObject * mpi_testall(MPIObject *self, PyObject *requests)
   if (ret != MPI_SUCCESS)
     {
       // We do not dare to release the buffers now!
-      PyErr_SetString(PyExc_RuntimeError, "MPI_Testall error occured.");
+      PyErr_SetString(PyExc_RuntimeError, "MPI_Testall error occurred.");
       return NULL;
     }
 #endif
@@ -510,7 +519,7 @@ static PyObject * mpi_waitall(MPIObject *self, PyObject *requests)
   if (ret != MPI_SUCCESS)
     {
       // We do not dare to release the buffers now!
-      PyErr_SetString(PyExc_RuntimeError, "MPI_Waitall error occured.");
+      PyErr_SetString(PyExc_RuntimeError, "MPI_Waitall error occurred.");
       return NULL;
     }
 #endif
@@ -530,7 +539,7 @@ static PyObject * mpi_waitall(MPIObject *self, PyObject *requests)
   free(rqs);
   Py_RETURN_NONE;
 }
- 
+
 
 static MPI_Datatype get_mpi_datatype(PyArrayObject *a)
 {
@@ -827,6 +836,78 @@ static PyObject * mpi_broadcast(MPIObject *self, PyObject *args)
   Py_RETURN_NONE;
 }
 
+static PyObject *mpi_compare(MPIObject *self, PyObject *args)
+{
+  MPIObject* other;
+  int result;
+  char* pyresult;
+  if (!PyArg_ParseTuple(args, "O", &other))
+    return NULL;
+
+  MPI_Comm_compare(self->comm, other->comm, &result);
+  if(result == MPI_IDENT) pyresult = "ident";
+  else if (result == MPI_CONGRUENT) pyresult = "congruent";
+  else if (result == MPI_SIMILAR) pyresult = "similar";
+  else if (result == MPI_UNEQUAL) pyresult = "unequal";
+  else return NULL;
+  return Py_BuildValue("s", pyresult);
+}
+
+static PyObject *mpi_translate_ranks(MPIObject *self, PyObject *args)
+{
+  PyObject* myranks_anytype; // Conversion to numpy array below
+  MPIObject* other;
+
+  if (!PyArg_ParseTuple(args, "OO", &other, &myranks_anytype))
+    return NULL;
+
+  // XXXXXX This uses NPY_LONG and NPY_INT.  On some computers the
+  // returned array is int32 while np.array(..., dtype=int) returns
+  // int64.  This should very probably be changed so it always
+  // corresponds to the default int of numpy.
+
+  // This handling of arrays of ranks is taken from the MPICommunicator
+  // creation method.  See that method for explanation of casting, datatypes
+  // etc.
+  PyArrayObject *myranks_long = (PyArrayObject*)PyArray_ContiguousFromAny(
+                                            myranks_anytype, NPY_LONG, 1, 1);
+  if(myranks_long == NULL)
+    return NULL;
+
+  int nranks = PyArray_DIM(myranks_long, 0);
+
+  PyArrayObject *myranks;
+  myranks = (PyArrayObject*)PyArray_Cast(myranks_long, NPY_INT);
+
+  npy_intp rankshape[1];
+  rankshape[0] = PyArray_SIZE(myranks);
+  PyArrayObject* other_ranks = (PyArrayObject*)PyArray_SimpleNew(1, rankshape,
+                                                                 NPY_INT);
+
+  MPI_Group mygroup, othergroup;
+  MPI_Comm_group(self->comm, &mygroup);
+  MPI_Comm_group(other->comm, &othergroup);
+
+  int* rankdata = (int*)PyArray_BYTES(myranks);
+  int* otherrankdata = (int*)PyArray_BYTES(other_ranks);
+  MPI_Group_translate_ranks(mygroup, nranks, rankdata, othergroup,
+                            otherrankdata);
+
+  // Return something with a definite value to Python.
+  for(int i=0; i < nranks; i++) {
+      if(otherrankdata[i] == MPI_UNDEFINED) {
+          otherrankdata[i] = -1;
+      }
+  }
+  PyObject* other_ranks_anytype = PyArray_Cast(other_ranks,
+      PyArray_TYPE((PyArrayObject*)myranks_anytype));
+
+  Py_DECREF(myranks_long);
+  Py_DECREF(myranks);
+  Py_DECREF(other_ranks);
+  return (PyObject*)other_ranks_anytype;
+}
+
 static PyObject * mpi_alltoallv(MPIObject *self, PyObject *args)
 {
   PyArrayObject* send_obj;
@@ -853,22 +934,29 @@ static PyObject * mpi_alltoallv(MPIObject *self, PyObject *args)
   int *r_displs = GPAW_MALLOC(int, self->size);
 
   /* Create count and displacement arrays in units of bytes */
-  int elem_size = PyArray_DESCR(send_obj)->elsize;
+  int elem_size = PyArray_ITEMSIZE(send_obj);
+
   long* tmp1 = PyArray_DATA(send_cnts);
   long* tmp2 = PyArray_DATA(send_displs);
   long* tmp3 = PyArray_DATA(recv_cnts);
   long* tmp4 = PyArray_DATA(recv_displs);
-  for (int i=0; i < self->size; i++)
-    {
+  for (int i=0; i < self->size; i++) {
       s_cnts[i] = tmp1[i] * elem_size;
       s_displs[i] = tmp2[i] * elem_size;
       r_cnts[i] = tmp3[i] * elem_size;
       r_displs[i] = tmp4[i] * elem_size;
-    }
+  }
 
-  MPI_Alltoallv(PyArray_BYTES(send_obj), s_cnts, s_displs,
+  MPI_Alltoallv(PyArray_BYTES(send_obj),
+                s_cnts, s_displs,
                 MPI_BYTE, PyArray_BYTES(recv_obj), r_cnts,
                 r_displs, MPI_BYTE, self->comm);
+
+  free(s_cnts);
+  free(s_displs);
+  free(r_cnts);
+  free(r_displs);
+
   Py_RETURN_NONE;
 }
 
@@ -946,6 +1034,10 @@ static PyMethodDef mpi_methods[] = {
      "alltoallv(sbuf, scnt, sdispl, rbuf, ...) send data from all tasks to all tasks."},
     {"broadcast",        (PyCFunction)mpi_broadcast,    METH_VARARGS,
      "broadcast(buffer, root) Broadcast data in-place from root task."},
+    {"compare",          (PyCFunction)mpi_compare,      METH_VARARGS,
+     "compare two communicators for identity using MPI_Comm_compare."},
+    {"translate_ranks",  (PyCFunction)mpi_translate_ranks, METH_VARARGS,
+     "figure out correspondence between ranks on two communicators."},
     {"get_members",      (PyCFunction)get_members,      METH_VARARGS, 0},
     {"get_c_object",     (PyCFunction)get_c_object,     METH_VARARGS, 0},
     {"new_communicator", (PyCFunction)MPICommunicator,  METH_VARARGS,
@@ -966,14 +1058,14 @@ static PyObject *NewMPIObject(PyTypeObject* type, PyObject *args,
 {
     static char *kwlist[] = {NULL};
     MPIObject* self;
-  
+
     if (! PyArg_ParseTupleAndKeywords(args, kwds, "", kwlist))
         return NULL;
 
     self = (MPIObject *) type->tp_alloc(type, 0);
     if (self == NULL)
         return NULL;
-   
+
 #   ifndef GPAW_INTERPRETER
         MPI_Init(NULL, NULL);
 #   endif
@@ -996,10 +1088,10 @@ static PyObject *NewMPIObject(PyTypeObject* type, PyObject *args,
 static int InitMPIObject(MPIObject* self, PyObject *args, PyObject *kwds)
 {
   static char *kwlist[] = {NULL};
-  
+
   if (! PyArg_ParseTupleAndKeywords(args, kwds, "", kwlist))
     return -1;
-  
+
   return 0;
 }
 
