@@ -7,7 +7,8 @@ from ase.optimize.test.test import (test_optimizer, all_optimizers,
 db1 = ase.db.connect('systems.db')
 db = ase.db.connect('results-emt.db', serial=True)
 
-systems = [row.toatoms() for row in db1.select() if row.formula != 'C5H12']
+systems = [(row.name, row.toatoms())
+           for row in db1.select() if row.formula != 'C5H12']
 
 for opt in all_optimizers:
     optimizer = get_optimizer(opt)
