@@ -12,7 +12,7 @@ from gpaw.fdtd.potential_couplers import (RefinerPotentialCoupler,
                                           MultipolesPotentialCoupler)
 from gpaw.grid_descriptor import GridDescriptor
 from gpaw.mpi import world, serial_comm
-from gpaw.poisson import PoissonSolver
+from gpaw.poisson import PoissonSolver, FDPoissonSolver
 from gpaw.tddft import TDDFT
 from gpaw.transformers import Transformer
 from gpaw.utilities.gpts import get_number_of_grid_points
@@ -204,7 +204,7 @@ class FDTDPoissonSolver:
         self.cl.extrapolated_qm_phi = None
         self.cl.dcomm = communicator
         self.cl.dparsize = None
-        self.qm = PoissonOrganizer(PoissonSolver)  # Default solver
+        self.qm = PoissonOrganizer(FDPoissonSolver)  # Default solver
         self.qm.spacing_def = qm_spacing * np.ones(3) / Bohr
         self.qm.cell = np.array(cell) / Bohr
 
@@ -249,7 +249,11 @@ class FDTDPoissonSolver:
 
     def estimate_memory(self, mem):
         # self.cl.poisson_solver.estimate_memory(mem)
-        self.qm.poisson_solver.estimate_memory(mem)
+        #print(self.qm.poisson_solver.estimate_memory.__code__.co_varnames)
+        # WTF?  How can this shabby method suddenly be unbound?  It needs both self and mem.
+        # Ferchrissakes!
+        #self.qm.poisson_solver.estimate_memory(mem=mem)
+        pass
 
     # Return the TDDFT stencil by default
     def get_stencil(self, mode='qm'):
@@ -271,6 +275,7 @@ class FDTDPoissonSolver:
 
         # Create quantum Poisson solver
         self.qm.poisson_solver = PoissonSolver(
+            name='fd',
             nn=self.nn,
             eps=self.eps,
             relax=self.relax,
@@ -285,6 +290,7 @@ class FDTDPoissonSolver:
 
         # Create classical PoissonSolver
         self.cl.poisson_solver = PoissonSolver(
+            name='fd',
             nn=self.nn,
             eps=self.eps,
             relax=self.relax,
@@ -869,7 +875,8 @@ class FDTDPoissonSolver:
               eps=None,
               maxcharge=1e-6,
               zero_initial_phi=False,
-              calculation_mode=None):
+              calculation_mode=None,
+              timer=None):
 
         if self.density is None:
             raise RuntimeError('FDTDPoissonSolver requires a density object.'
