@@ -41,7 +41,7 @@ def check_fft_size(n):
 
 def get_efficient_fft_size(N, n=1):
     """Return smallest efficient fft size.
-    
+
     Must be greater than or equal to N and divisible by n.
     """
     N = -(-N // n) * n
@@ -72,6 +72,8 @@ class FFTWPlan:
         lib.fftw_execute(self.plan)
 
     def __del__(self, lib=lib):
+        # We keep a reference to lib so that it's not GC'ed before we have
+        # cleaned up.
         lib.fftw_destroy_plan(self.plan)
 
 
@@ -110,23 +112,35 @@ if lib is None:
     FFTPlan = NumpyFFTPlan
 else:
     FFTPlan = FFTWPlan
+
+    lib.fftw_execute.argtypes = [ctypes.c_void_p]
+    lib.fftw_destroy_plan.argtypes = [ctypes.c_void_p]
+
     lib.fftw_plan_dft_3d.argtypes = [
         ctypes.c_int, ctypes.c_int, ctypes.c_int,
         np.ctypeslib.ndpointer(dtype=complex, ndim=3, flags='C_CONTIGUOUS'),
         np.ctypeslib.ndpointer(dtype=complex, ndim=3, flags='C_CONTIGUOUS'),
         ctypes.c_int, ctypes.c_uint]
+    lib.fftw_plan_dft_3d.restype = ctypes.c_void_p
+
     lib.fftw_plan_dft_r2c_3d.argtypes = [
         ctypes.c_int, ctypes.c_int, ctypes.c_int,
         np.ctypeslib.ndpointer(dtype=float, ndim=3),
         np.ctypeslib.ndpointer(dtype=complex, ndim=3, flags='C_CONTIGUOUS'),
         ctypes.c_uint]
+    lib.fftw_plan_dft_r2c_3d.restype = ctypes.c_void_p
+
     lib.fftw_plan_dft_c2r_3d.argtypes = [
         ctypes.c_int, ctypes.c_int, ctypes.c_int,
         np.ctypeslib.ndpointer(dtype=complex, ndim=3, flags='C_CONTIGUOUS'),
         np.ctypeslib.ndpointer(dtype=float, ndim=3),
         ctypes.c_uint]
+    lib.fftw_plan_dft_c2r_3d.restype = ctypes.c_void_p
+
     try:
         lib.fftw_plan_with_nthreads.argtypes = [ctypes.c_int]
-        assert lib.fftw_init_threads()
     except AttributeError:
-        pass
+        pass  # We have a single-threaded FFTW
+    else:
+        if not lib.fftw_init_threads():
+            raise RuntimeError('fftw_init_threads failed')
