@@ -8,6 +8,7 @@ from math import pi, sqrt
 import numpy as np
 from ase.units import Bohr, Hartree, alpha
 from ase.parallel import paropen
+from ase.utils import basestring
 
 import gpaw.mpi as mpi
 from gpaw.utilities import packed_index
@@ -52,17 +53,21 @@ class KSSingles(ExcitationList):
                  txt=None):
 
         self.eps = None
+        self.world = mpi.world
 
-        if isinstance(calculator, str):
-            filehandle = open(calculator)
+        self.calculator = None
+        if isinstance(calculator, basestring):
+            self.read(calculator)
+            return self.select(eps=eps, istart=istart, jend=jend,
+                               energy_range=energy_range)
         if filehandle is not None:
-            self.world = mpi.world
-            self.read(fh=filehandle, istart=istart, jend=jend)
-            return None
+            self.read(fh=filehandle)
+            return self.select(eps, istart=istart, jend=jend,
+                               energy_range=energy_range)
 
         # LCAO calculation requires special actions
         if calculator is not None:
-            self.lcao = calculator.input_parameters.mode == 'lcao'
+            self.lcao = calculator.parameters.mode == 'lcao'
 
         ExcitationList.__init__(self, calculator, txt=txt)
 
@@ -79,30 +84,21 @@ class KSSingles(ExcitationList):
         self.world = calculator.wfs.world
 
         # parallelization over bands not yet supported
-        assert(calculator.wfs.band_comm.size == 1)
+        assert(calculator.wfs.bd.comm.size == 1)
 
         self.select(nspins, eps, istart, jend, energy_range)
 
         trkm = self.get_trk()
+        print(file=self.txt)
+        print('KSS %d transitions' % len(self), file=self.txt)
         print('KSS TRK sum %g (%g,%g,%g)' %
               (np.sum(trkm) / 3., trkm[0], trkm[1], trkm[2]), file=self.txt)
         pol = self.get_polarizabilities(lmax=3)
         print('KSS polarisabilities(l=0-3) %g, %g, %g, %g' %
               tuple(pol.tolist()), file=self.txt)
 
-    def select(self, nspins=None, eps=0.001,
-               istart=0, jend=sys.maxsize, energy_range=None):
-        """Select KSSingles according to the given criterium."""
-
-        paw = self.calculator
-        wfs = paw.wfs
-        self.dtype = wfs.dtype
-        self.kpt_u = wfs.kpt_u
-
-        if not self.lcao and self.kpt_u[0].psit_nG is None:
-            raise RuntimeError('No wave functions in calculator!')
-
-        # criteria
+    @staticmethod
+    def emin_emax(energy_range):
         emin = -sys.float_info.max
         emax = sys.float_info.max
         if energy_range is not None:
@@ -112,10 +108,35 @@ class KSSingles(ExcitationList):
                 emax /= Hartree
             except:
                 emax = energy_range / Hartree
+        return emin, emax
+
+    def select(self, nspins=None, eps=0.001,
+               istart=0, jend=sys.maxsize, energy_range=None):
+        """Select KSSingles according to the given criterium."""
+
+        # criteria
+        emin, emax = self.emin_emax(energy_range)
         self.istart = istart
         self.jend = jend
         self.eps = eps
 
+        if self.calculator is None:  # I'm read from a file
+            # throw away all not needed entries
+            for i, ks in reversed(list(enumerate(self))):
+                if ((ks.fij / ks.weight) <= eps or
+                    ks.i < istart or ks.j > jend or
+                    ks.energy < emin or ks.energy > emax):
+                    del(self[i])
+            return None
+
+        paw = self.calculator
+        wfs = paw.wfs
+        self.dtype = wfs.dtype
+        self.kpt_u = wfs.kpt_u
+
+        if not self.lcao and self.kpt_u[0].psit_nG is None:
+            raise RuntimeError('No wave functions in calculator!')
+
         # here, we need to take care of the spins also for
         # closed shell systems (Sz=0)
         # vspin is the virtual spin of the wave functions,
@@ -147,7 +168,7 @@ class KSSingles(ExcitationList):
                     kpt = self.kpt_u[myks]
                     for i in range(nbands):
                         for j in range(i + 1, nbands):
-                            fij = kpt.f_n[i] - kpt.f_n[j]
+                            fij = (kpt.f_n[i] - kpt.f_n[j]) / kpt.weight
                             epsij = kpt.eps_n[j] - kpt.eps_n[i]
                             if (fij > eps and
                                 epsij >= emin and epsij < emax and
@@ -181,22 +202,32 @@ class KSSingles(ExcitationList):
         for kss in self:
             kss.distribute()
 
-    def read(self, filename=None, fh=None, istart=0, jend=sys.maxsize):
+    def read(self, filename=None, fh=None):
         """Read myself from a file"""
+        def fail(f):
+            raise RuntimeError(f.name + ' does not contain ' +
+                               self.__class__.__name__ + ' data')
         if fh is None:
             if filename.endswith('.gz'):
                 import gzip
-                f = gzip.open(filename)
+                f = gzip.open(filename, 'rt')
             else:
                 f = open(filename, 'r')
+
+            # there can be other information, i.e. the LrTDDFT header
+            try:
+                content = f.read()
+                f.seek(content.index('# KSSingles'))
+                del(content)
+                f.readline()
+            except ValueError:
+                fail(f)
         else:
             f = fh
+            # we assume to be at the right place and read the header
+            if not f.readline().strip() == '# KSSingles':
+                fail(f)
 
-        try:
-            assert(f.readline().strip() == '# KSSingles')
-        except:
-            raise RuntimeError(f.name + ' is not a ' +
-                               self.__class__.__name__ + ' data file')
         words = f.readline().split()
         n = int(words[0])
         if len(words) == 1:
@@ -211,9 +242,8 @@ class KSSingles(ExcitationList):
         self.npspins = 1
         for i in range(n):
             kss = KSSingle(string=f.readline(), dtype=self.dtype)
-            if (kss.i >= istart) and (kss.j <= jend):
-                self.append(kss)
-                self.npspins = max(self.npspins, kss.pspin + 1)
+            self.append(kss)
+            self.npspins = max(self.npspins, kss.pspin + 1)
         self.update()
 
         if fh is None:
@@ -285,7 +315,7 @@ class KSSingles(ExcitationList):
         if fh is None:
             if filename.endswith('.gz') and mpi.rank == mpi.MASTER:
                 import gzip
-                f = gzip.open(filename, 'wb')
+                f = gzip.open(filename, 'wt')
             else:
                 f = paropen(filename, 'w')
         else:
@@ -296,10 +326,35 @@ class KSSingles(ExcitationList):
         f.write('{0}\n'.format(self.eps))
         for kss in self:
             f.write(kss.outstring())
-
         if fh is None:
             f.close()
 
+    def overlap(self, ov_nn, other):
+        """Matrix element overlaps determined from wave function overlaps.
+
+        Parameters
+        ----------
+        ov_nn: array
+            Wave function overlap factors from a displaced calculator.
+            Index 0 corresponds to our own wavefunctions conjugated and
+            index 1 to the others' wavefunctions
+
+        Returns
+        -------
+        ov_pp: array
+            Overlap corresponding to matrix elements.
+            Index 0 corresponds to our own matrix elements conjugated and
+            index 1 to the others' matrix elements
+        """
+        n0 = len(self)
+        n1 = len(other)
+        ov_pp = np.zeros((n0, n1), dtype=ov_nn.dtype)
+        i1_p = [ex.i for ex in other]
+        j1_p = [ex.j for ex in other]
+        for p0, ex0 in enumerate(self):
+            ov_pp[p0, :] = ov_nn[ex0.i, i1_p].conj() * ov_nn[ex0.j, j1_p]
+        return ov_pp
+
 
 class KSSingle(Excitation, PairDensity):
 
@@ -580,6 +635,17 @@ class KSSingle(Excitation, PairDensity):
             string += ')'
         return string
 
+    def __eq__(self, other):
+        """KSSingles are considred equal when their indices are equal."""
+        return (self.pspin == other.pspin and self.k == other.k and
+                self.i == other.i and self.j == other.j)
+
+    def __hash__(self):
+        """Hash similar to __eq__"""
+        if not hasattr(self, 'hash'):
+            self.hash = hash((self.spin, self.k, self.i, self.j))
+        return self.hash
+
     #
     # User interface: ##
     #
