@@ -3,7 +3,7 @@ from math import sqrt
 
 import numpy as np
 from ase.units import Hartree
-from ase.utils import convert_string_to_fd
+from ase.utils import convert_string_to_fd, basestring
 from ase.utils.timing import Timer
 
 import gpaw.mpi as mpi
@@ -53,13 +53,12 @@ class OmegaMatrix:
             eh_comm = mpi.serial_comm
 
         self.eh_comm = eh_comm
+        self.fullkss = kss
 
         if filehandle is not None:
-            self.kss = kss
             self.read(fh=filehandle)
             return None
 
-        self.fullkss = kss
         self.finegrid = finegrid
 
         if calculator is None:
@@ -90,7 +89,10 @@ class OmegaMatrix:
         if xc == 'RPA':
             xc = None  # enable RPA as keyword
         if xc is not None:
-            self.xc = XC(xc)
+            if isinstance(xc, basestring):
+                self.xc = XC(xc)
+            else:
+                self.xc = xc
             self.xc.initialize(self.paw.density, self.paw.hamiltonian,
                                wfs, self.paw.occupations)
 
@@ -365,7 +367,7 @@ class OmegaMatrix:
                       self.time_left(timer, t0, ij, nij), file=self.txt)
 
     def Coulomb_integral_kss(self, kss_ij, kss_kq, phit, rhot,
-                             timer=None):
+                             timer=None, yukawa=False):
         # smooth part
         if timer:
             timer.start('integrate')
@@ -389,7 +391,11 @@ class OmegaMatrix:
             Pq_i = Pkq_ani[a][kss_kq.j]
             Dkq_ii = np.outer(Pk_i, Pq_i)
             Dkq_p = pack(Dkq_ii)
-            C_pp = wfs.setups[a].M_pp
+            if yukawa and hasattr(self.xc, 'omega') and self.xc.omega > 0:
+                C_pp = wfs.setups[a].calculate_yukawa_interaction(
+                    self.xc.omega)
+            else:
+                C_pp = wfs.setups[a].M_pp
             #   ----
             # 2 >      P   P  C    P  P
             #   ----    ip  jr prst ks qt
@@ -531,7 +537,13 @@ class OmegaMatrix:
         return self.timestring(t0 * (nij - ij - 1) + t)
 
     def get_map(self, istart=None, jend=None, energy_range=None):
-        """Return the reduction map for the given requirements"""
+        """Return the reduction map for the given requirements
+
+        Returns
+        -------
+        map - list of original indices
+        kss - reduced KSSingles object
+        """
 
         self.istart = istart
         self.jend = jend
@@ -598,14 +610,16 @@ class OmegaMatrix:
         self.kss = kss
         diagonalize(self.eigenvectors, self.eigenvalues)
 
-    def Kss(self, kss=None):
-        """Set and get own Kohn-Sham singles"""
-        if kss is not None:
-            self.fullkss = kss
-        if(hasattr(self, 'fullkss')):
-            return self.fullkss
-        else:
-            return None
+    @property
+    def kss(self):
+        if hasattr(self, '_kss'):
+            return self._kss
+        return self.fullkss
+
+    @kss.setter
+    def kss(self, kss):
+        """Set current (restricted) KSSingles object"""
+        self._kss = kss
 
     def read(self, filename=None, fh=None):
         """Read myself from a file"""
