@@ -1,11 +1,11 @@
-# Copyright (C) 2003  CAMP
+#Copyright (C) 2003  CAMP
 # Please see the accompanying LICENSE file for further information.
 
 import warnings
 from math import pi
 
 import numpy as np
-from numpy.fft import fftn, ifftn, fft2, ifft2
+from numpy.fft import fftn, ifftn, fft2, ifft2, rfft2, irfft2, fft, ifft
 
 from gpaw import PoissonConvergenceError
 from gpaw.dipole_correction import DipoleCorrection
@@ -17,12 +17,14 @@ from gpaw.utilities.gauss import Gaussian
 from gpaw.utilities.grid import grid2grid
 from gpaw.utilities.ewald import madelung
 from gpaw.utilities.tools import construct_reciprocal
+from gpaw.utilities.timing import NullTimer
 import _gpaw
 
+POISSON_GRID_WARNING = """Grid unsuitable for FDPoissonSolver!
 
-POISSON_GRID_WARNING = """Grid unsuitable for Poisson solver!
+Consider using FastPoissonSolver instead.
 
-The Poisson solver does not have sufficient multigrid levels for good
+The FDPoissonSolver does not have sufficient multigrid levels for good
 performance and will converge inefficiently if at all, or yield wrong
 results.
 
@@ -42,7 +44,7 @@ number of multigrid levels even if the total number of grid points
 is divisible by a high power of 2."""
 
 
-def create_poisson_solver(name='fd', **kwargs):
+def create_poisson_solver(name='fast', **kwargs):
     if isinstance(name, _PoissonSolver):
         return name
     elif isinstance(name, dict):
@@ -54,19 +56,25 @@ def create_poisson_solver(name='fd', **kwargs):
         from gpaw.fdtd.poisson_fdtd import FDTDPoissonSolver
         return FDTDPoissonSolver(**kwargs)
     elif name == 'fd':
-        return PoissonSolver(**kwargs)
+        return FDPoissonSolverWrapper(**kwargs)
+    elif name == 'fast':
+        return FastPoissonSolver(**kwargs)
     elif name == 'ExtraVacuumPoissonSolver':
         from gpaw.poisson_extravacuum import ExtraVacuumPoissonSolver
         return ExtraVacuumPoissonSolver(**kwargs)
     else:
         raise ValueError('Unknown poisson solver: %s' % name)
 
-
-def PoissonSolver(dipolelayer=None, **kwargs):
+def PoissonSolver(name='fast', dipolelayer=None, **kwargs):
+    p = create_poisson_solver(name=name, **kwargs)
     if dipolelayer is not None:
-        return DipoleCorrection(PoissonSolver(**kwargs), dipolelayer)
-    return FDPoissonSolver(**kwargs)
+        p = DipoleCorrection(p, dipolelayer)
+    return p
 
+def FDPoissonSolverWrapper(dipolelayer=None, **kwargs):
+    if dipolelayer is not None:
+        return DipoleCorrection(FDPoissonSolver(**kwargs), dipolelayer)
+    return FDPoissonSolver(**kwargs)
 
 class _PoissonSolver(object):
     """Abstract PoissonSolver class
@@ -123,7 +131,7 @@ class BasePoissonSolver(_PoissonSolver):
         return '\n'.join(lines)
 
     def solve(self, phi, rho, charge=None, eps=None, maxcharge=1e-6,
-              zero_initial_phi=False):
+              zero_initial_phi=False, timer=NullTimer()):
         self._init()
         assert np.all(phi.shape == self.gd.n_c)
         assert np.all(rho.shape == self.gd.n_c)
@@ -146,7 +154,7 @@ class BasePoissonSolver(_PoissonSolver):
             for phi_cor in phi_cor_L:
                 phi -= phi_cor
 
-            niter = self.solve_neutral(phi, rho_neutral, eps=eps)
+            niter = self.solve_neutral(phi, rho_neutral, eps=eps, timer=timer)
             # correct error introduced by removing multipoles
             for phi_cor in phi_cor_L:
                 phi += phi_cor
@@ -156,7 +164,7 @@ class BasePoissonSolver(_PoissonSolver):
             charge = actual_charge
         if abs(charge) <= maxcharge:
             # System is charge neutral. Use standard solver
-            return self.solve_neutral(phi, rho - background, eps=eps)
+            return self.solve_neutral(phi, rho - background, eps=eps, timer=timer)
 
         elif abs(charge) > maxcharge and self.gd.pbc_c.all():
             # System is charged and periodic. Subtract a homogeneous
@@ -166,7 +174,7 @@ class BasePoissonSolver(_PoissonSolver):
             if zero_initial_phi:
                 phi[:] = 0.0
 
-            iters = self.solve_neutral(phi, rho - background, eps=eps)
+            iters = self.solve_neutral(phi, rho - background, eps=eps, timer=timer)
             return iters
 
         elif abs(charge) > maxcharge and not self.gd.pbc_c.any():
@@ -214,7 +222,7 @@ class BasePoissonSolver(_PoissonSolver):
                 axpy(-q, self.phi_gauss, phi)  # phi -= q * self.phi_gauss
 
             # Determine potential from neutral density using standard solver
-            niter = self.solve_neutral(phi, rho_neutral, eps=eps)
+            niter = self.solve_neutral(phi, rho_neutral, eps=eps, timer=timer)
 
             # correct error introduced by removing monopole
             axpy(q, self.phi_gauss, phi)  # phi += q * self.phi_gauss
@@ -233,6 +241,7 @@ class BasePoissonSolver(_PoissonSolver):
             self.phi_gauss = gauss.get_gauss_pot(0)
 
 
+
 class FDPoissonSolver(BasePoissonSolver):
     def __init__(self, nn=3, relax='J', eps=2e-10, maxiter=1000,
                  remove_moment=None, use_charge_center=False):
@@ -381,7 +390,7 @@ class FDPoissonSolver(BasePoissonSolver):
         self.postsmooths[level] = 8
         self._initialized = True
 
-    def solve_neutral(self, phi, rho, eps=2e-10):
+    def solve_neutral(self, phi, rho, eps=2e-10, timer=None):
         self._init()
         self.phis[0] = phi
 
@@ -473,7 +482,7 @@ class NoInteractionPoissonSolver(_PoissonSolver):
     def get_stencil(self):
         return 1
 
-    def solve(self, phi, rho, charge):
+    def solve(self, phi, rho, charge, timer=None):
         return 0
 
     def set_grid_descriptor(self, gd):
@@ -519,11 +528,11 @@ class FFTPoissonSolver(BasePoissonSolver):
             return
 
         gd = self.grids[-1]
-        k2_Q, N3 = construct_reciprocal(gd, distributed=True)
+        k2_Q, N3 = construct_reciprocal(gd)
         self.poisson_factor_Q = 4.0 * np.pi / k2_Q
         self._initialized = True
 
-    def solve_neutral(self, phi_g, rho_g, eps=None):
+    def solve_neutral(self, phi_g, rho_g, eps=None, timer=None):
         self._init()
         # Will be a bit more efficient if reduced dimension is always
         # contiguous.  Probably more things can be improved...
@@ -628,7 +637,7 @@ class FixedBoundaryPoissonSolver(FDPoissonSolver):
         self.comm_reshape = not (self.gd.parsize_c[0] == 1 and
                                  self.gd.parsize_c[1] == 1)
 
-    def solve(self, phi_g, rho_g, charge=None):
+    def solve(self, phi_g, rho_g, charge=None, timer=None):
         if charge is None:
             actual_charge = self.gd.integrate(rho_g)
         else:
@@ -705,7 +714,7 @@ class FixedBoundaryPoissonSolver(FDPoissonSolver):
             global_phi_g = None
         return global_phi_g
 
-    def solve_neutral(self, phi_g, rho_g):
+    def solve_neutral(self, phi_g, rho_g, timer=None):
         # b_phi1 and b_phi2 are the boundary Hartree potential values
         # of left and right sides
 
@@ -755,3 +764,383 @@ class FixedBoundaryPoissonSolver(FDPoissonSolver):
             self.gd.distribute(global_phi_g, phi_g)
         else:
             phi_g[:] = phi_g3
+
+"""def rfst2(A_g, axes=[0,1]):
+    assert axes[0] == 0
+    assert axes[1] == 1
+    x,y,z = A_g.shape
+    temp_g = np.zeros((x*2+2, y*2+2, z))
+    temp_g[1:x+1, 1:y+1,:] = A_g
+    temp_g[x+2:, 1:y+1,:] = -A_g[::-1, :, :]
+    temp_g[1:x+1, y+2:,:] = -A_g[:, ::-1, :]
+    temp_g[x+2:, y+2:,:] = A_g[::-1, ::-1, :]
+    X = -4*rfft2(temp_g, axes=axes)
+    return X[1:x+1, 1:y+1, :].real
+
+def irfst2(A_g, axes=[0,1]):
+    assert axes[0] == 0
+    assert axes[1] == 1
+    x,y,z = A_g.shape
+    temp_g = np.zeros((x*2+2, (y*2+2)//2+1, z))
+    temp_g[1:x+1, 1:y+1,:] = A_g
+    temp_g[x+2:, 1:y+1,:] = -A_g[::-1, :, :]
+    return -0.25*irfft2(temp_g, axes=axes)[1:x+1, 1:y+1, :].real
+"""
+
+
+from scipy.fftpack import dst as scipydst
+
+use_scipy_transforms = True
+
+
+def rfst2(A_g, axes=[0,1]):
+    all = set([0,1,2])
+    third = [ all.difference(set(axes)).pop() ]
+
+    if use_scipy_transforms:
+        Y = A_g
+        for axis in axes:
+            Y = scipydst(Y, axis=axis, type=1)
+        Y *= 2**len(axes)
+        return Y
+
+    A_g = np.transpose(A_g, axes + third)
+    x,y,z = A_g.shape
+    temp_g = np.zeros((x*2+2, y*2+2, z))
+    temp_g[1:x+1, 1:y+1,:] = A_g
+    temp_g[x+2:, 1:y+1,:] = -A_g[::-1, :, :]
+    temp_g[1:x+1, y+2:,:] = -A_g[:, ::-1, :]
+    temp_g[x+2:, y+2:,:] = A_g[::-1, ::-1, :]
+    X = -4*rfft2(temp_g, axes=[0,1])[1:x+1, 1:y+1, :].real
+    return np.transpose(X, np.argsort(axes + third))
+
+def irfst2(A_g, axes=[0,1]):
+    if use_scipy_transforms:
+        Y = A_g
+        for axis in axes:
+            Y = scipydst(Y, axis=axis, type=1)
+        magic = 1.0 / (16 * np.prod([A_g.shape[axis] + 1 for axis in axes]))
+        Y *= magic
+        #Y /= 211200
+        return Y
+
+    all = set([0,1,2])
+    third = [ all.difference(set(axes)).pop() ]
+    A_g = np.transpose(A_g, axes + third)
+    x,y,z = A_g.shape
+    temp_g = np.zeros((x*2+2, (y*2+2)//2+1, z))
+    temp_g[1:x+1, 1:y+1,:] = A_g.real
+    temp_g[x+2:, 1:y+1,:] = -A_g[::-1, :, :].real
+    X = -0.25*irfft2(temp_g, axes=[0,1])[1:x+1, 1:y+1, :]
+
+    T = np.transpose(X, np.argsort(axes + third))
+    return T
+
+
+# This method needs to be taken from fftw / scipy to gain speedup of ~4x
+def fst(A_g, axis):
+    x, y, z = A_g.shape
+    N_c = np.array([x, y, z])
+    N_c[axis] = N_c[axis]*2 + 2
+    temp_g = np.zeros(N_c, dtype=A_g.dtype)
+    if axis == 0:
+        temp_g[1:x+1, :,:] = A_g
+        temp_g[x+2:, :,:] = -A_g[::-1, :, :]
+    elif axis == 1:
+        temp_g[:, 1:y+1,:] = A_g
+        temp_g[:, y+2:, :] = -A_g[:, ::-1, :]
+    elif axis == 2:
+        temp_g[:, :, 1:z+1] = A_g
+        temp_g[:, :, z+2:] = -A_g[:, ::, ::-1]
+    else:
+        raise NotImplementedError()
+    X = 0.5j*fft(temp_g, axis=axis)
+    if axis == 0:
+        return X[1:x+1, :, :]
+    elif axis == 1:
+        return X[:, 1:y+1, :]
+    elif axis == 2:
+        return X[:, :, 1:z+1]
+
+def ifst(A_g, axis):
+    x, y, z = A_g.shape
+    N_c = np.array([x, y, z])
+    N_c[axis] = N_c[axis]*2 + 2
+    temp_g = np.zeros(N_c, dtype=A_g.dtype)
+
+    if axis == 0:
+        temp_g[1:x+1, :,:] = A_g
+        temp_g[x+2:, :,:] = -A_g[::-1, :, :]
+    elif axis == 1:
+        temp_g[:, 1:y+1,:] = A_g
+        temp_g[:, y+2:, :] = -A_g[:, ::-1, :]
+    elif axis == 2:
+        temp_g[:, :, 1:z+1] = A_g
+        temp_g[:, :, z+2:] = -A_g[:, ::, ::-1]
+    else:
+        raise NotImplementedError()
+
+    X_g = ifft(temp_g, axis=axis)
+    if axis == 0:
+        return -2j*X_g[1:x+1, :, :]
+    elif axis == 1:
+        return -2j*X_g[:, 1:y+1, :]
+    elif axis == 2:
+        return -2j*X_g[:, :, 1:z+1]
+
+
+def transform(A_g, axis=None, pbc=True):
+    if pbc:
+        return fft(A_g, axis=axis)
+    else:
+        if not use_scipy_transforms:
+            x = fst(A_g, axis)
+            return x
+        y = scipydst(A_g, axis=axis, type=1)
+        y *= .5
+        return y
+
+def transform2(A_g, axes=None, pbc=[True, True]):
+    if all(pbc):
+        return fft2(A_g, axes=axes)
+    elif not any(pbc):
+        return rfst2(A_g, axes=axes)
+    else:
+        return transform(transform(A_g, axis=axes[0], pbc=pbc[0]),
+                         axis=axes[1], pbc=pbc[1])
+
+def itransform(A_g, axis=None, pbc=True):
+    if pbc:
+        return ifft(A_g, axis=axis)
+    else:
+        if not use_scipy_transforms:
+            x = ifst(A_g, axis)
+            return x
+        y = scipydst(A_g, axis=axis, type=1)
+        magic = 1.0 / (A_g.shape[axis] + 1)
+        y *= magic
+        return y
+
+def itransform2(A_g, axes=None, pbc=[True, True]):
+    if all(pbc):
+        return ifft2(A_g, axes=axes)
+    elif not any(pbc):
+        return irfst2(A_g, axes=axes)
+    else:
+        return itransform(itransform(A_g, axis=axes[0], pbc=pbc[0]),
+                          axis=axes[1], pbc=pbc[1])
+
+
+class BadAxesError(ValueError):
+    pass
+
+
+class FastPoissonSolver(BasePoissonSolver):
+    def __init__(self, nn=3,  **kwargs):
+        BasePoissonSolver.__init__(self, **kwargs)
+        self.nn = nn
+        # We may later enable this to work with Cholesky, but not now:
+        self.use_cholesky = False
+
+    def _init(self):
+        pass
+
+    def set_grid_descriptor(self, gd):
+        self.gd = gd
+        axes = np.arange(3)
+        pbc_c = np.array(gd.pbc_c, dtype=bool)
+        periodic_axes = axes[pbc_c]
+        non_periodic_axes = axes[np.logical_not(pbc_c)]
+
+
+        # Find out which axes are orthogonal (0, 1 or 3)
+        # Note that one expects that the axes are always rotated in
+        # conventional form, thus for all axes to be
+        # classified as orthogonal, the cell_cv needs to be diagonal.
+        # This may always be achieved by rotating
+        # the unit-cell along with the atoms. The classification is
+        # inherited from grid_descriptor.orthogonal.
+        dotprods = np.dot(gd.cell_cv, gd.cell_cv.T)
+        # For each direction, check whether there is only one nonzero
+        # element in that row (necessarily being the diagonal element,
+        # since this is a cell vector length and must be > 0).
+        orthogonal_c = (np.abs(dotprods) > 1e-10).sum(axis=0) == 1
+        assert sum(orthogonal_c) in [0, 1, 3]
+
+        non_orthogonal_axes = axes[np.logical_not(orthogonal_c)]
+
+        if not all(pbc_c | orthogonal_c):
+            raise BadAxesError('Each axis must be periodic or orthogonal '
+                               'to other axes.  But we have pbc={} '
+                               'and orthogonal={}'
+                               .format(pbc_c.astype(int),
+                                       orthogonal_c.astype(int)))
+
+        # We sort them, and pick the longest non-periodic axes as the
+        # cholesky axis.
+        sorted_non_periodic_axes = sorted(non_periodic_axes,
+                                          key=lambda c: gd.N_c[c])
+        if self.use_cholesky:
+            if len(sorted_non_periodic_axes) > 0:
+                cholesky_axes = [ sorted_non_periodic_axes[-1] ]
+                if cholesky_axes[0] in non_orthogonal_axes:
+                    msg = ('Cholesky axis cannot be non-orthogonal. '
+                           'Do you really want a non-orthogonal non-periodic '
+                           'axis? If so, run with use_cholesky=False.')
+                    raise NotImplementedError(msg)
+                fst_axes = sorted_non_periodic_axes[0:-1]
+            else:
+                cholesky_axes = []
+                fst_axes = []
+        else:
+            cholesky_axes = []
+            fst_axes = sorted_non_periodic_axes
+        fft_axes = list(periodic_axes)
+
+        (self.cholesky_axes, self.fst_axes,
+         self.fft_axes) = cholesky_axes, fst_axes, fft_axes
+
+        fftfst_axes = self.fft_axes + self.fst_axes
+        axes = self.fft_axes + self.fst_axes + self.cholesky_axes
+        self.axes = axes
+        gd_x = [ self.gd ]
+
+        # Create xy flat decomposition (where x=axes[0] and y=axes[1])
+        domain = gd.N_c.copy()
+        domain[axes[0]] = 1
+        domain[axes[1]] = 1
+        parsize_c = decompose_domain(domain, gd.comm.size)
+        gd_x.append(gd.new_descriptor(parsize_c=parsize_c))
+
+        # Create z flat decomposition
+        domain = gd.N_c.copy()
+        domain[axes[2]] = 1
+        parsize_c = decompose_domain(domain, gd.comm.size)
+        gd_x.append(gd.new_descriptor(parsize_c=parsize_c))
+        self.gd_x = gd_x
+
+        # Calculate eigenvalues in fst/fft decomposition for
+        # non-cholesky axes in parallel
+        r_cx = np.indices(gd_x[-1].n_c)
+        r_cx += gd_x[-1].beg_c[:,np.newaxis, np.newaxis, np.newaxis]
+        r_cx = r_cx.astype(complex)
+        for c, axis in enumerate(fftfst_axes):
+            r_cx[axis] *= 2j * np.pi / gd_x[-1].N_c[axis]
+            if axis in fst_axes:
+                r_cx[axis] /= 2
+        for c, axis in enumerate(cholesky_axes):
+            r_cx[axis] = 0.0
+        np.exp(r_cx, out=r_cx)
+        fft_lambdas = np.zeros_like(r_cx[0], dtype=complex)
+        laplace = Laplace(gd_x[-1], -0.25 / pi, self.nn)
+        self.stencil_description = laplace.description
+
+        for coeff, offset_c in zip(laplace.coef_p, laplace.offset_pc):
+            offset_c = np.array(offset_c)
+            if not any(offset_c):
+                # The centerpoint is handled with (temp-1.0)
+                continue
+            non_zero_axes, = np.where(offset_c)
+            if set(non_zero_axes).issubset(fftfst_axes):
+                temp = np.ones_like(fft_lambdas)
+                for c, axis in enumerate(fftfst_axes):
+                    temp *= r_cx[axis] ** offset_c[axis]
+                fft_lambdas += coeff * (temp - 1.0)
+
+        assert np.linalg.norm(fft_lambdas.imag) < 1e-10
+        fft_lambdas = fft_lambdas.real.copy()  # arr.real is not contiguous
+
+        # If there is no Cholesky decomposition, the system is already
+        # fully diagonal and we can directly invert the linear problem
+        # by dividing with the eigenvalues.
+        assert len(cholesky_axes) == 0
+        # if len(cholesky_axes) == 0:
+        with np.errstate(divide='ignore'):
+            self.inv_fft_lambdas = np.where(
+                np.abs(fft_lambdas) > 1e-10, 1.0 / fft_lambdas, 0)
+
+    def solve_neutral(self, phi_g, rho_g, eps=None, timer=None):
+        gd1 = self.gd
+        work1_g = rho_g
+
+        for c in range(2):
+            # There are two decompositions, xy-flat and then z-flat
+            timer.start('Communicate fwd %d' % c)
+            gd2 = self.gd_x[c + 1]
+            work2_g = gd2.empty(dtype=work1_g.dtype)
+            grid2grid(gd1.comm, gd1, gd2, work1_g, work2_g)
+            timer.stop('Communicate fwd %d' % c)
+            if c == 0:
+               timer.start('fft2')
+               work1_g = transform2(work2_g, axes=self.axes[:2],
+                                    pbc=gd1.pbc_c[self.axes[:2]])
+               timer.stop('fft2')
+            elif c == 1:
+                if len(self.cholesky_axes) == 0:
+                    # The remaining problem is 0D dimensional, i.e the
+                    # problem has been fully diagonalized
+                    timer.start('fft')
+                    work1_g = transform(work2_g, axis=self.axes[2],
+                                        pbc=gd1.pbc_c[self.axes[2]])
+                    timer.stop('fft')
+                else:
+                    raise NotImplementedError
+            else:
+                raise NotImplementedError
+            gd1 = gd2
+
+        if len(self.cholesky_axes) == 0:
+            # The remaining problem is 0D dimensional, i.e the problem
+            # has been fully diagonalized
+            work1_g *= self.inv_fft_lambdas
+        else:
+            assert len(self.cholesky_axes) == 1
+            #axis = self.cholesky_axes[0]
+            raise NotImplementedError
+
+        for c in [1, 0]:
+            gd2 = self.gd_x[c]
+
+            if c == 0:
+               timer.start('fft2')
+               work2_g = itransform2(work1_g, axes=self.axes[1::-1],
+                                     pbc=gd1.pbc_c[self.axes[1::-1]])
+               timer.stop('fft2')
+            elif c == 1:
+                if len(self.cholesky_axes) == 0:
+                    # The remaining problem is 0D dimensional, i.e the
+                    # problem has been fully diagonalized
+                    timer.start('fft')
+                    work2_g = itransform(work1_g, axis=self.axes[2],
+                                         pbc=gd1.pbc_c[self.axes[2]])
+                    timer.stop('fft')
+                else:
+                    raise NotImplementedError
+            else:
+                raise NotImplementedError
+
+            timer.start('Communicate bwd %d' % c)
+            work1_g = gd2.empty(dtype=work2_g.dtype)
+            grid2grid(gd1.comm, gd1, gd2, work2_g, work1_g)
+            timer.stop('Communicate bwd %d' % c)
+            gd1 = gd2
+
+        phi_g[:] = work1_g.real
+        return 1 # Non-iterative method, return 1 iteration
+
+    def todict(self):
+        d = super(FastPoissonSolver, self).todict()
+        d.update({'name': 'fast', 'nn': self.nn})
+        return d
+
+    def estimate_memory(self, mem):
+        pass
+
+    def get_description(self):
+        return """\
+FastPoissonSolver using
+    %s stencil;
+    FFT axes: %s;
+    FST axes: %s.
+""" % (self.stencil_description,
+       self.fft_axes, self.fst_axes)
