@@ -30,13 +30,20 @@ class Preconditioner:
     def calculate_kinetic_energy(self, psit_xG, kpt):
         return None
 
-    def __call__(self, residuals, kpt, ekin=None):
+    def __call__(self, residuals, kpt, ekin=None, out=None):
         if residuals.ndim == 3:
-            return self.__call__(residuals[np.newaxis], kpt)[0]
+            if out is None:
+                return self.__call__(residuals[np.newaxis], kpt)[0]
+            return self.__call__(residuals[np.newaxis], kpt,
+                                 out=out[np.newaxis])[0]
         nb = len(residuals)  # number of bands
         phases = kpt.phase_cd
         step = self.step
-        d0, q0 = self.scratch0[:, :nb]
+        if out is None:
+            d0, q0 = self.scratch0[:, :nb]
+        else:
+            d0 = out
+            q0 = self.scratch0[0, :nb]
         r1, d1, q1 = self.scratch1[:, :nb]
         r2, d2, q2 = self.scratch2[:, :nb]
         self.restrictor0(-residuals, r1, phases)
