@@ -1,8 +1,6 @@
 from __future__ import print_function
-import numpy as np
 
 from ase import Atom, Atoms
-from ase.parallel import rank
 
 from gpaw import GPAW
 from gpaw.test import equal
@@ -13,15 +11,15 @@ try:
     e = NaCl.get_potential_energy()
     niter = None
 except IOError:
-    h = 0.21 # gridspacing
-    a = [6.5, 6.5, 7.7] # unit cell
-    d = 2.3608 # experimental bond length
+    h = 0.21  # gridspacing
+    a = [6.5, 6.5, 7.7]  # unit cell
+    d = 2.3608  # experimental bond length
 
     NaCl = Atoms([Atom('Na', [0, 0, 0]),
                   Atom('Cl', [0, 0, d])],
                  pbc=False, cell=a)
     NaCl.center()
-    calc = GPAW(h=h, xc='LDA', nbands=5, lmax=0,
+    calc = GPAW(h=h, xc='LDA', nbands=5,
                 setups={'Na': '1'},
                 convergence={'eigenstates': 1e-6}, spinpol=1)
 
@@ -30,6 +28,8 @@ except IOError:
     niter = calc.get_number_of_iterations()
     calc.write('NaCl.gpw')
 
+equal(e, -4.9066211328768246, 0.001)
+
 dv = NaCl.get_volume() / calc.get_number_of_grid_points().prod()
 nt1 = calc.get_pseudo_density(gridrefinement=1)
 Zt1 = nt1.sum() * dv
@@ -43,7 +43,3 @@ for gridrefinement in [1, 2, 4]:
     Z = n.sum() * dv / gridrefinement**3
     print('Integral of all-electron density:', Z)
     equal(Z, 28, 1e-5)
-
-energy_tolerance = 0.0004
-niter_tolerance = 0
-equal(e, -4.908677, energy_tolerance)
