@@ -2,7 +2,7 @@ from __future__ import print_function
 
 import numpy as np
 from gpaw.spline import Spline
-from gpaw.poisson import PoissonSolver, FFTPoissonSolver
+from gpaw.poisson import FDPoissonSolver, FFTPoissonSolver, PoissonSolver
 from gpaw.grid_descriptor import GridDescriptor
 from gpaw.lfc import LocalizedFunctionsCollection as LFC
 
@@ -25,18 +25,19 @@ def f(n, p):
 
     I = gd.integrate(a)
     b = gd.zeros()
-    p.solve(b, a, charge=0)#, eps=1e-20)
+    p.solve(b, a, charge=0, eps=1e-20)
     return gd.collect(b, broadcast=1)
 
-b1 = f(8, PoissonSolver(nn=1, relax='J'))
-b2 = f(8, PoissonSolver(nn=1, relax='GS'))
-
+b1 = f(8, FDPoissonSolver(nn=1, relax='J'))
+b2 = f(8, FDPoissonSolver(nn=1, relax='GS'))
+b3 = f(8, PoissonSolver(nn=1))
 err1 = abs(b1[0,0,0]-b1[8,8,8])
 err2 = abs(b2[0,0,0]-b2[8,8,8])
-
+err3 = abs(b1[0,0,0]-b3[8,8,8])
 print(err1)
 print(err2)
-
+print(err3)
+assert err3 < 1e-10
 assert err1 < 6e-16
 assert err2 < 3e-6 # XXX Shouldn't this be better?
 
