@@ -3,8 +3,7 @@
 
 from __future__ import print_function
 from ase.build import molecule
-from gpaw import GPAW, PoissonSolver, Davidson, Mixer
-from gpaw.utilities import h2gpts
+from gpaw import GPAW, Davidson, Mixer
 from gpaw.test.pseudopotential.H_sg15 import pp_text
 from gpaw import setup_paths
 setup_paths.insert(0, '.')
@@ -26,18 +25,14 @@ system.center(vacuum=2.5)
 def getkwargs():
     return dict(eigensolver=Davidson(4),
                 mixer=Mixer(0.8, 5, 10.0),
-                xc='oldPBE',
-                poissonsolver=PoissonSolver(relax='GS', eps=1e-8))
+                xc='oldPBE')
 
-calc1 = GPAW(setups='sg15',
-             gpts=h2gpts(0.13, system.get_cell(), idiv=8),
-             **getkwargs())
+calc1 = GPAW(setups='sg15', h=0.13, **getkwargs())
 system.set_calculator(calc1)
 system.get_potential_energy()
 eps1 = calc1.get_eigenvalues()
 
-calc2 = GPAW(gpts=h2gpts(0.2, system.get_cell(), idiv=8),
-             **getkwargs())
+calc2 = GPAW(h=0.2, **getkwargs())
 system.set_calculator(calc2)
 system.get_potential_energy()
 eps2 = calc2.get_eigenvalues()
