@@ -1,13 +1,11 @@
-from __future__ import print_function
 import numpy as np
 
 from gpaw.test import equal
+from gpaw.fftw import FFTPlan
 from gpaw.grid_descriptor import GridDescriptor
 from gpaw.spline import Spline
 import gpaw.mpi as mpi
-from gpaw.lfc import LocalizedFunctionsCollection as LFC
 from gpaw.wavefunctions.pw import PWDescriptor, PWLFC
-from gpaw.kpt_descriptor import KPointDescriptor
 
 
 x = 2.0
@@ -23,7 +21,6 @@ spos_ac = np.array([(0.15, 0.45, 0.95)])
 pd = PWDescriptor(45, gd, complex)
 pdr = PWDescriptor(45, gd)
 
-from gpaw.fftw import FFTPlan
 print(FFTPlan)
 
 for l in range(4):
@@ -32,7 +29,7 @@ for l in range(4):
 
     lfc = PWLFC([[s]], pd)
     lfcr = PWLFC([[s]], pdr)
-    
+
     c_axi = {0: np.zeros((1, 2 * l + 1), complex)}
     c_axi[0][0, 0] = 1.9
     cr_axi = {0: np.zeros((1, 2 * l + 1))}
@@ -48,7 +45,7 @@ for l in range(4):
 
     a = pd.ifft(b)
     ar = pdr.ifft(br)
-    equal(abs(a-ar).max(), 0, 1e-14)
+    equal(abs(a - ar).max(), 0, 1e-14)
 
     if l == 0:
         a = a[:, ::-1].copy()
@@ -57,10 +54,11 @@ for l in range(4):
 
     lfc.integrate(b0, c_axi)
     lfcr.integrate(br0, cr_axi)
-    assert abs(c_axi[0][0]-cr_axi[0][0]).max() < 1e-14
+
+    assert abs(c_axi[0][0] - cr_axi[0][0]).max() < 1e-14
 
     c_axiv = {0: np.zeros((1, 2 * l + 1, 3), complex)}
     cr_axiv = {0: np.zeros((1, 2 * l + 1, 3))}
     lfc.derivative(b0, c_axiv)
     lfcr.derivative(br0, cr_axiv)
-    assert abs(c_axiv[0][0]-cr_axiv[0][0]).max() < 1e-14
+    assert abs(c_axiv[0][0] - cr_axiv[0][0]).max() < 1e-14
