@@ -9,22 +9,23 @@ QNA = {'alpha': 2.0,
        'setup_name': 'PBE',
        'type': 'qna-gga'}
 
-atoms = BodyCenteredCubic(symbol='Fe', latticeconstant=2.854,
-                          pbc=(1,1,1))
+atoms = BodyCenteredCubic(symbol='Fe',
+                          latticeconstant=2.854,
+                          pbc=(1, 1, 1))
 
-atoms.set_initial_magnetic_moments([2,2])
+atoms.set_initial_magnetic_moments([2, 2])
 
 calc = GPAW(mode=PW(400),
-            kpts=(3,3,3),
+            kpts=(3, 3, 3),
+            experimental={'niter_fixdensity': 2},
             xc=QNA,
-            txt='qna_spinpol.txt',
-            )
+            parallel={'domain': 1},
+            txt='qna_spinpol.txt')
 
 atoms.set_calculator(calc)
 atoms.get_potential_energy()
 magmoms = atoms.get_magnetic_moments()
 
-tol = 1e-5
-equal(2.253246, magmoms[0], tol)
-equal(2.253246, magmoms[1], tol)
-
+tol = 0.003
+equal(2.243, magmoms[0], tol)
+equal(2.243, magmoms[1], tol)
