@@ -4,8 +4,9 @@ from ase.units import Bohr
 from gpaw.lfc import BasisFunctions
 from gpaw.utilities import unpack
 from gpaw.utilities.tools import tri2full
-from gpaw import debug
-from gpaw.lcao.overlap import NewTwoCenterIntegrals as NewTCI
+#from gpaw import debug
+#from gpaw.lcao.overlap import NewTwoCenterIntegrals as NewTCI
+from gpaw.lcao.tci import TCIExpansions
 from gpaw.utilities.blas import gemm, gemmdot
 from gpaw.wavefunctions.base import WaveFunctions
 from gpaw.lcao.atomic_correction import get_atomic_correction
@@ -32,6 +33,32 @@ class LCAO(Mode):
         return dct
 
 
+def update_phases(C_unM, q_u, ibzk_qc, spos_ac, oldspos_ac, setups, Mstart):
+    """Complex-rotate coefficients compensating discontinuous phase shift.
+
+    This changes the coefficients to counteract the phase discontinuity
+    of overlaps when atoms move across a cell boundary."""
+
+    # We don't want to apply any phase shift unless we crossed a cell
+    # boundary.  So we round the shift to either 0 or 1.
+    #
+    # Example: spos_ac goes from 0.01 to 0.99 -- this rounds to 1 and
+    # we apply the phase.  If someone moves an atom by half a cell
+    # without crossing a boundary, then we are out of luck.  But they
+    # should have reinitialized from LCAO anyway.
+    phase_qa = np.exp(2j * np.pi *
+                      np.dot(ibzk_qc, (spos_ac - oldspos_ac).T.round()))
+
+    for q, C_nM in zip(q_u, C_unM):
+        if C_nM is None:
+            continue
+        for a in range(len(spos_ac)):
+            M1 = setups.M_a[a] - Mstart
+            M2 = M1 + setups[a].nao
+            M1 = max(0, M1)
+            C_nM[:, M1:M2] *= phase_qa[q, a]  # (may truncate M2)
+
+
 # replace by class to make data structure perhaps a bit less confusing
 def get_r_and_offsets(nl, spos_ac, cell_cv):
     r_and_offset_aao = {}
@@ -67,19 +94,20 @@ class LCAOWaveFunctions(WaveFunctions):
         self.S_qMM = None
         self.T_qMM = None
         self.P_aqMi = None
+        self.debug_tci = False
 
         if atomic_correction is None:
             if ksl.using_blacs:
-                atomic_correction = 'distributed'
+                atomic_correction = 'scipy'
             else:
                 atomic_correction = 'dense'
         if isinstance(atomic_correction, str):
             atomic_correction = get_atomic_correction(atomic_correction)
         self.atomic_correction = atomic_correction
 
-        self.timer.start('TCI: Evaluate splines')
-        self.tci = NewTCI(gd.cell_cv, gd.pbc_c, setups, kd.ibzk_qc, kd.gamma)
-        self.timer.stop('TCI: Evaluate splines')
+        #self.tci = NewTCI(gd.cell_cv, gd.pbc_c, setups, kd.ibzk_qc, kd.gamma)
+        with self.timer('TCI: Evaluate splines'):
+            self.tciexpansions = TCIExpansions.new_from_setups(setups)
 
         self.basis_functions = BasisFunctions(gd,
                                               [setup.phit_j
@@ -114,6 +142,7 @@ class LCAOWaveFunctions(WaveFunctions):
                                    self.ksl)
 
     def set_positions(self, spos_ac, atom_partition=None, move_wfs=False):
+        oldspos_ac = self.spos_ac
         with self.timer('Basic WFS set positions'):
             WaveFunctions.set_positions(self, spos_ac, atom_partition)
 
@@ -131,47 +160,91 @@ class LCAOWaveFunctions(WaveFunctions):
         Mstart = self.ksl.Mstart
         mynao = Mstop - Mstart
 
-        if self.ksl.using_blacs:  # XXX
+        #if self.ksl.using_blacs:  # XXX
             # S and T have been distributed to a layout with blacs, so
             # discard them to force reallocation from scratch.
             #
             # TODO: evaluate S and T when they *are* distributed, thus saving
             # memory and avoiding this problem
-            self.S_qMM = None
-            self.T_qMM = None
+        for kpt in self.kpt_u:
+            kpt.S_MM = None
+            kpt.T_MM = None
 
-        S_qMM = self.S_qMM
-        T_qMM = self.T_qMM
+        # Free memory in case of old matrices:
+        self.S_qMM = self.T_qMM = self.P_aqMi = None
 
-        if S_qMM is None:  # XXX
-            # First time:
-            assert T_qMM is None
-            if self.ksl.using_blacs:  # XXX
-                self.tci.set_matrix_distribution(Mstart, mynao)
-
-            S_qMM = np.empty((nq, mynao, nao), self.dtype)
-            T_qMM = np.empty((nq, mynao, nao), self.dtype)
+        if self.dtype == complex and oldspos_ac is not None:
+            update_phases([kpt.C_nM for kpt in self.kpt_u],
+                          [kpt.q for kpt in self.kpt_u],
+                          self.kd.ibzk_qc, spos_ac, oldspos_ac,
+                          self.setups, Mstart)
 
         for kpt in self.kpt_u:
             if kpt.C_nM is None:
                 kpt.C_nM = np.empty((mynbands, nao), self.dtype)
 
-        self.P_aqMi = {}
-        for a in self.basis_functions.my_atom_indices:
-            ni = self.setups[a].ni
-            self.P_aqMi[a] = np.empty((nq, nao, ni), self.dtype)
+        if 0:#self.debug_tci:
+            #if self.ksl.using_blacs:
+            #    self.tci.set_matrix_distribution(Mstart, mynao)
+            oldS_qMM = np.empty((nq, mynao, nao), self.dtype)
+            oldT_qMM = np.empty((nq, mynao, nao), self.dtype)
 
-        self.timer.start('TCI: Calculate S, T, P')
-        # Calculate lower triangle of S and T matrices:
-        self.tci.calculate(spos_ac, S_qMM, T_qMM, self.P_aqMi)
 
-        if self.atomic_correction.name != 'dense':
-            from gpaw.lcao.newoverlap import newoverlap
-            self.P_neighbors_a, self.P_aaqim = newoverlap(self, spos_ac)
+            oldP_aqMi = {}
+            for a in self.basis_functions.my_atom_indices:
+                ni = self.setups[a].ni
+                oldP_aqMi[a] = np.empty((nq, nao, ni), self.dtype)
+
+            # Calculate lower triangle of S and T matrices:
+            self.timer.start('tci calculate')
+            #self.tci.calculate(spos_ac, oldS_qMM, oldT_qMM,
+            #                   oldP_aqMi)
+            self.timer.stop('tci calculate')
+
+
+        self.timer.start('mktci')
+        manytci = self.tciexpansions.get_manytci_calculator(
+            self.setups, self.gd, spos_ac, self.kd.ibzk_qc, self.dtype,
+            self.timer)
+        self.timer.stop('mktci')
+        self.manytci = manytci
+        self.newtci = manytci.tci
+
+        my_atom_indices = self.basis_functions.my_atom_indices
+        self.timer.start('ST tci')
+        newS_qMM, newT_qMM = manytci.O_qMM_T_qMM(self.gd.comm,
+                                                 Mstart, Mstop,
+                                                 self.ksl.using_blacs)
+        self.timer.stop('ST tci')
+        self.timer.start('P tci')
+        P_qIM = manytci.P_qIM(my_atom_indices)
+        self.timer.stop('P tci')
+        self.P_aqMi = newP_aqMi = manytci.P_aqMi(my_atom_indices)
+        self.P_qIM = P_qIM  # XXX atomic correction
+
+        # TODO
+        #   OK complex/conj, periodic images
+        #   OK scalapack
+        #   derivatives/forces
+        #   sparse
+        #   use symmetry/conj tricks to reduce calculations
+        #   enable caching of spherical harmonics
+
+        #if self.atomic_correction.name != 'dense':
+        #from gpaw.lcao.newoverlap import newoverlap
+        #self.P_neighbors_a, self.P_aaqim = newoverlap(self, spos_ac)
         self.atomic_correction.gobble_data(self)
 
-        self.atomic_correction.add_overlap_correction(self, S_qMM)
-        self.timer.stop('TCI: Calculate S, T, P')
+        #if self.atomic_correction.name == 'scipy':
+        #    Pold_qIM = self.atomic_correction.Psparse_qIM
+        #    for q in range(nq):
+        #        maxerr = abs(Pold_qIM[q] - P_qIM[q]).max()
+        #        print('sparse maxerr', maxerr)
+        #        assert maxerr == 0
+
+        self.atomic_correction.add_overlap_correction(self, newS_qMM)
+        if self.debug_tci:
+            self.atomic_correction.add_overlap_correction(self, oldS_qMM)
 
         if self.atomic_correction.implements_distributed_projections():
             my_atom_indices = self.atomic_correction.get_a_values()
@@ -179,29 +252,61 @@ class LCAOWaveFunctions(WaveFunctions):
             my_atom_indices = self.basis_functions.my_atom_indices
         self.allocate_arrays_for_projections(my_atom_indices)
 
-        S_MM = None  # allow garbage collection of old S_qMM after redist
-        S_qMM = self.ksl.distribute_overlap_matrix(S_qMM, root=-1)
-        T_qMM = self.ksl.distribute_overlap_matrix(T_qMM, root=-1)
+        #S_MM = None  # allow garbage collection of old S_qMM after redist
+        if self.debug_tci:
+            oldS_qMM = self.ksl.distribute_overlap_matrix(oldS_qMM, root=-1)
+            oldT_qMM = self.ksl.distribute_overlap_matrix(oldT_qMM, root=-1)
+
+        newS_qMM = self.ksl.distribute_overlap_matrix(newS_qMM, root=-1)
+        newT_qMM = self.ksl.distribute_overlap_matrix(newT_qMM, root=-1)
+
+        #if (debug and self.bd.comm.size == 1 and self.gd.comm.rank == 0 and
+        #    nao > 0 and not self.ksl.using_blacs):
+            # S and T are summed only on comm master, so check only there
+        #    from numpy.linalg import eigvalsh
+        #    self.timer.start('Check positive definiteness')
+        #    for S_MM in S_qMM:
+        #        tri2full(S_MM, UL='L')
+        #        smin = eigvalsh(S_MM).real.min()
+        #        if smin < 0:
+        #            raise RuntimeError('Overlap matrix has negative '
+        #                               'eigenvalue: %e' % smin)
+        #    self.timer.stop('Check positive definiteness')
+        self.positions_set = True
+
+        if self.debug_tci:
+            Serr = np.abs(newS_qMM - oldS_qMM).max()
+            Terr = np.abs(newT_qMM - oldT_qMM).max()
+            print('S maxerr', Serr)
+            print('T maxerr', Terr)
+            try:
+                assert Terr < 1e-15, Terr
+            except AssertionError:
+                np.set_printoptions(precision=6)
+                if self.world.rank == 0:
+                    print(newT_qMM)
+                    print(oldT_qMM)
+                    print(newT_qMM - oldT_qMM)
+                raise
+            assert Serr < 1e-15, Serr
+
+            assert len(oldP_aqMi) == len(newP_aqMi)
+            for a in oldP_aqMi:
+                Perr = np.abs(oldP_aqMi[a] - newP_aqMi[a]).max()
+                assert Perr < 1e-15, (a, Perr)
+
         for kpt in self.kpt_u:
             q = kpt.q
-            kpt.S_MM = S_qMM[q]
-            kpt.T_MM = T_qMM[q]
+            kpt.S_MM = newS_qMM[q]
+            kpt.T_MM = newT_qMM[q]
+        self.S_qMM = newS_qMM
+        self.T_qMM = newT_qMM
 
-        if (debug and self.bd.comm.size == 1 and self.gd.comm.rank == 0 and
-            nao > 0 and not self.ksl.using_blacs):
-            # S and T are summed only on comm master, so check only there
-            from numpy.linalg import eigvalsh
-            self.timer.start('Check positive definiteness')
-            for S_MM in S_qMM:
-                tri2full(S_MM, UL='L')
-                smin = eigvalsh(S_MM).real.min()
-                if smin < 0:
-                    raise RuntimeError('Overlap matrix has negative '
-                                       'eigenvalue: %e' % smin)
-            self.timer.stop('Check positive definiteness')
-        self.positions_set = True
-        self.S_qMM = S_qMM
-        self.T_qMM = T_qMM
+        # Elpa wants to reuse the decomposed form of S_qMM.
+        # We need to keep track of the existence of that object here,
+        # since this is where we change S_qMM.  Hence, expect this to
+        # become arrays after the first diagonalization:
+        self.decomposed_S_qMM = [None] * len(self.S_qMM)
 
     def initialize(self, density, hamiltonian, spos_ac):
         # Note: The above line exists also in set_positions.
@@ -338,9 +443,9 @@ class LCAOWaveFunctions(WaveFunctions):
         ksl = self.ksl
         nao = ksl.nao
         mynao = ksl.mynao
-        nq = len(self.kd.ibzk_qc)
         dtype = self.dtype
-        tci = self.tci
+        #tci = self.tci
+        newtci = self.newtci
         gd = self.gd
         bfs = self.basis_functions
 
@@ -352,14 +457,13 @@ class LCAOWaveFunctions(WaveFunctions):
 
         if not isblacs:
             self.timer.start('TCI derivative')
-            dThetadR_qvMM = np.empty((nq, 3, mynao, nao), dtype)
-            dTdR_qvMM = np.empty((nq, 3, mynao, nao), dtype)
-            dPdR_aqvMi = {}
-            for a in self.basis_functions.my_atom_indices:
-                ni = self.setups[a].ni
-                dPdR_aqvMi[a] = np.empty((nq, 3, nao, ni), dtype)
-            tci.calculate_derivative(self.spos_ac, dThetadR_qvMM, dTdR_qvMM,
-                                     dPdR_aqvMi)
+
+            dThetadR_qvMM, dTdR_qvMM = self.manytci.O_qMM_T_qMM(
+                gd.comm, Mstart, Mstop, False, derivative=True)
+
+            dPdR_aqvMi = self.manytci.P_aqMi(
+                self.basis_functions.my_atom_indices, derivative=True)
+
             gd.comm.sum(dThetadR_qvMM)
             gd.comm.sum(dTdR_qvMM)
             self.timer.stop('TCI derivative')
@@ -448,10 +552,10 @@ class LCAOWaveFunctions(WaveFunctions):
                 rho_mm = redistributor.redistribute(rho1_mm)
                 return rho_mm
 
-            pcutoff_a = [max([pt.get_cutoff() for pt in setup.pt_j])
-                         for setup in self.setups]
-            phicutoff_a = [max([phit.get_cutoff() for phit in setup.phit_j])
-                           for setup in self.setups]
+            #pcutoff_a = [max([pt.get_cutoff() for pt in setup.pt_j])
+            #             for setup in self.setups]
+            #phicutoff_a = [max([phit.get_cutoff() for phit in setup.phit_j])
+            #               for setup in self.setups]
 
             # XXX should probably use bdsize x gdsize instead
             # That would be consistent with some existing grids
@@ -512,6 +616,7 @@ class LCAOWaveFunctions(WaveFunctions):
             for u, kpt in enumerate(self.kpt_u):
                 dEdTrhoT_vMM = (dTdR_qvMM[kpt.q] *
                                 rhoT_uMM[u][np.newaxis]).real
+                # XXX load distribution!
                 for a, M1, M2 in my_slices():
                     Fkin_av[a, :] += \
                         2.0 * dEdTrhoT_vMM[:, M1:M2].sum(-1).sum(-1)
@@ -536,27 +641,12 @@ class LCAOWaveFunctions(WaveFunctions):
             del dThetadRE_vMM
 
         if isblacs:
-            from gpaw.lcao.overlap import TwoCenterIntegralCalculator
+            #from gpaw.lcao.overlap import TwoCenterIntegralCalculator
             self.timer.start('Prepare TCI loop')
             M_a = bfs.M_a
-
             Fkin2_av = np.zeros_like(F_av)
             Ftheta2_av = np.zeros_like(F_av)
-
-            overlapcalc = TwoCenterIntegralCalculator(self.kd.ibzk_qc,
-                                                      derivative=False)
-
-            # XXX this is not parallel *AT ALL*.
-            self.timer.start('Get neighbors')
-            nl = tci.atompairs.pairs.neighbors
-            r_and_offset_aao = get_r_and_offsets(nl, self.spos_ac, gd.cell_cv)
-            atompairs = sorted(r_and_offset_aao.keys())
-            self.timer.stop('Get neighbors')
-
-            T_expansions = tci.T_expansions
-            Theta_expansions = tci.Theta_expansions
-            P_expansions = tci.P_expansions
-            nq = len(self.kd.ibzk_qc)
+            atompairs = self.newtci.a1a2.get_atompairs()
 
             self.timer.start('broadcast dH')
             alldH_asp = {}
@@ -582,33 +672,6 @@ class LCAOWaveFunctions(WaveFunctions):
             #
             # To this end we will loop over all pairs of atoms (a1, a3),
             # and then a sub-loop over (a3, a2).
-            from gpaw.lcao.overlap import DerivativeAtomicDisplacement
-
-            class Displacement(DerivativeAtomicDisplacement):
-                def __init__(self, a1, a2, R_c, offset):
-                    phases = overlapcalc.phaseclass(overlapcalc.ibzk_qc,
-                                                    offset)
-                    DerivativeAtomicDisplacement.__init__(self, None, a1, a2,
-                                                          R_c, offset, phases)
-
-            # Cache of Displacement objects with spherical harmonics with
-            # evaluated spherical harmonics.
-            disp_aao = {}
-
-            def get_displacements(a1, a2, maxdistance):
-                # XXX the way maxdistance is handled it can lead to
-                # bad caching when different maxdistances are passed
-                # to subsequent calls with same pair of atoms
-                disp_o = disp_aao.get((a1, a2))
-                if disp_o is None:
-                    disp_o = []
-                    for R_c, offset in r_and_offset_aao[(a1, a2)]:
-                        if np.linalg.norm(R_c) > maxdistance:
-                            continue
-                        disp = Displacement(a1, a2, R_c, offset)
-                        disp_o.append(disp)
-                    disp_aao[(a1, a2)] = disp_o
-                return [disp for disp in disp_o if disp.r < maxdistance]
 
             self.timer.stop('Prepare TCI loop')
             self.timer.start('Not so complicated loop')
@@ -626,9 +689,8 @@ class LCAOWaveFunctions(WaveFunctions):
                 if m1start >= blocksize1 or m2start >= blocksize2:
                     continue  # (we have only one block per CPU)
 
-                T_expansion = T_expansions.get(a1, a2)
-                Theta_expansion = Theta_expansions.get(a1, a2)
-                nm1, nm2 = T_expansion.shape
+                nm1 = self.setups[a1].nao
+                nm2 = self.setups[a2].nao
 
                 m1stop = min(m1start + nm1, m1max)
                 m2stop = min(m2start + nm2, m2max)
@@ -643,14 +705,7 @@ class LCAOWaveFunctions(WaveFunctions):
                 M1stop = J1start + m1stop - m1start
                 J2stop = J2start + m2stop - m2start
 
-                dTdR_qvmm = T_expansion.zeros((nq, 3), dtype=dtype)
-                dThetadR_qvmm = Theta_expansion.zeros((nq, 3), dtype=dtype)
-
-                disp_o = get_displacements(a1, a2,
-                                           phicutoff_a[a1] + phicutoff_a[a2])
-                for disp in disp_o:
-                    disp.evaluate_overlap(T_expansion, dTdR_qvmm)
-                    disp.evaluate_overlap(Theta_expansion, dThetadR_qvmm)
+                dThetadR_qvmm, dTdR_qvmm = newtci.dOdR_dTdR(a1, a2)
 
                 for u, kpt in enumerate(self.kpt_u):
                     rhoT_mm = rhoT_umm[u][m1start:m1stop, m2start:m2stop]
@@ -690,42 +745,31 @@ class LCAOWaveFunctions(WaveFunctions):
                 if m1start >= blocksize1:
                     continue
 
-                P_expansion = P_expansions.get(a1, a3)
-                nm1 = P_expansion.shape[0]
+                nm1 = self.setups[a1].nao
                 m1stop = min(m1start + nm1, m1max)
                 if m1stop <= 0:
                     continue
 
+                dPdR_qvim = newtci.dPdR(a3, a1)
+                if dPdR_qvim is None:
+                    continue
+
+                dPdR_qvmi = -dPdR_qvim.transpose(0, 1, 3, 2).conj()
+
                 m1start = max(m1start, 0)
                 J1start = max(0, M1start - M_a[a1])
                 J1stop = J1start + m1stop - m1start
-
-                disp_o = get_displacements(a1, a3,
-                                           phicutoff_a[a1] + pcutoff_a[a3])
-                if len(disp_o) == 0:
-                    continue
-
-                dPdR_qvmi = P_expansion.zeros((nq, 3), dtype=dtype)
-                for disp in disp_o:
-                    disp.evaluate_overlap(P_expansion, dPdR_qvmi)
-
                 dPdR_qvmi = dPdR_qvmi[:, :, J1start:J1stop, :].copy()
                 for a2 in a2values[a3]:
                     m2start = M_a[a2] - M2start
                     if m2start >= blocksize2:
                         continue
 
-                    P_expansion2 = P_expansions.get(a2, a3)
-                    nm2 = P_expansion2.shape[0]
+                    nm2 = self.setups[a2].nao
                     m2stop = min(m2start + nm2, m2max)
                     if m2stop <= 0:
                         continue
 
-                    disp_o = get_displacements(a2, a3,
-                                               phicutoff_a[a2] + pcutoff_a[a3])
-                    if len(disp_o) == 0:
-                        continue
-
                     m2start = max(m2start, 0)
                     J2start = max(0, M2start - M_a[a2])
                     J2stop = J2start + m2stop - m2start
@@ -733,9 +777,10 @@ class LCAOWaveFunctions(WaveFunctions):
                     if (a2, a3) in dHP_and_dSP_aauim:
                         dHP_uim, dSP_uim = dHP_and_dSP_aauim[(a2, a3)]
                     else:
-                        P_qmi = P_expansion2.zeros((nq,), dtype=dtype)
-                        for disp in disp_o:
-                            disp.evaluate_direct(P_expansion2, P_qmi)
+                        P_qim = newtci.P(a3, a2)
+                        if P_qim is None:
+                            continue
+                        P_qmi = P_qim.transpose(0, 2, 1).conj()
                         P_qmi = P_qmi[:, J2start:J2stop].copy()
                         dH_sp = alldH_asp[a3]
                         dS_ii = self.setups[a3].dO_ii
@@ -745,7 +790,8 @@ class LCAOWaveFunctions(WaveFunctions):
                         for u, kpt in enumerate(self.kpt_u):
                             dH_ii = unpack(dH_sp[kpt.s])
                             dHP_im = np.dot(P_qmi[kpt.q], dH_ii).T.conj()
-                            # XXX only need nq of these
+                            # XXX only need nq of these,
+                            # but the looping is over all u
                             dSP_im = np.dot(P_qmi[kpt.q], dS_ii).T.conj()
                             dHP_uim.append(dHP_im)
                             dSP_uim.append(dSP_im)
@@ -925,7 +971,7 @@ class LCAOWaveFunctions(WaveFunctions):
         nM1, nM2 = self.ksl.get_overlap_matrix_shape()
         mem.subnode('S, T [2 x qmm]', 2 * nq * nM1 * nM2 * itemsize)
         mem.subnode('P [aqMi]', nq * nao * ni_total // self.gd.comm.size)
-        self.tci.estimate_memory(mem.subnode('TCI'))
+        #self.tci.estimate_memory(mem.subnode('TCI'))
         self.basis_functions.estimate_memory(mem.subnode('BasisFunctions'))
         self.eigensolver.estimate_memory(mem.subnode('Eigensolver'),
                                          self.dtype)
