@@ -58,9 +58,14 @@ def XC(kernel, parameters=None, atoms=None, collinear=True):
                     'C09-vdW', 'mBEEF-vdW', 'BEEF-vdW']:
             from gpaw.xc.vdw import VDWFunctional
             return VDWFunctional(name, **kwargs)
-        elif name in ['EXX', 'PBE0', 'B3LYP']:
+        elif name in ['EXX', 'PBE0', 'B3LYP',
+                      'CAMY-BLYP', 'CAMY-B3LYP', 'LCY-BLYP', 'LCY-PBE']:
             from gpaw.xc.hybrid import HybridXC
             return HybridXC(name, **kwargs)
+        elif name.startswith('LCY-') or name.startswith('CAMY-'):
+            parts = name.split('(')
+            from gpaw.xc.hybrid import HybridXC
+            return HybridXC(parts[0], omega=float(parts[1][:-1]))
         elif name in ['HSE03', 'HSE06']:
             from gpaw.xc.exx import EXX
             return EXX(name, **kwargs)
@@ -119,7 +124,8 @@ def XC(kernel, parameters=None, atoms=None, collinear=True):
             kernel = XCNull()
         elif name == 'QNA':
             from gpaw.xc.qna import QNA
-            return QNA(atoms, kernel['parameters'], kernel['setup_name'], alpha=kernel['alpha'])
+            return QNA(atoms, kernel['parameters'], kernel['setup_name'],
+                       alpha=kernel['alpha'], stencil=kwargs.get('stencil', 2))
         else:
             kernel = LibXC(name)
 
